import { __decorate, __metadata } from 'tslib';
import { isDevMode, TemplateRef, ViewContainerRef, Input, Directive, Component, NgModule, ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { isOn, set } from 'feature-toggle-service';
import { CommonModule } from '@angular/common';
import { Router } from '@angular/router';

var FeatureToggleDirective = /** @class */ (function () {
    function FeatureToggleDirective(templateRef, viewContainer) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.isOn = false;
    }
    FeatureToggleDirective.prototype.ngOnInit = function () {
        if (!this.featureToggle) {
            throw new Error('Attribute `featureToggle` should not be null or empty');
        }
        this.shouldRender();
    };
    FeatureToggleDirective.prototype.ngDoCheck = function () {
        if (this.isOn !== this.isOnCheck(this.featureToggle)) {
            this.shouldRender();
        }
    };
    FeatureToggleDirective.prototype.shouldRender = function () {
        this.isOn = this.isOnCheck(this.featureToggle);
        if (this.isOn) {
            this.viewContainer.createEmbeddedView(this.templateRef);
        }
        else {
            this.viewContainer.clear();
        }
    };
    FeatureToggleDirective.prototype.isOnCheck = function (featureToggle) {
        if (typeof featureToggle !== 'string' && !Array.isArray(featureToggle)) {
            if (isDevMode()) {
                console.error('`NgxFeatureToggle`: `featureToggle` should receive an array or an string as a value.');
            }
            return false;
        }
        return [].concat(featureToggle).every(function (toggle) {
            return toggle[0] === '!' ? !isOn(toggle.replace('!', '')) : isOn(toggle);
        });
    };
    FeatureToggleDirective.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ViewContainerRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FeatureToggleDirective.prototype, "featureToggle", void 0);
    FeatureToggleDirective = __decorate([
        Directive({
            // tslint:disable-next-line: directive-selector
            selector: '[featureToggle]',
        }),
        __metadata("design:paramtypes", [TemplateRef,
            ViewContainerRef])
    ], FeatureToggleDirective);
    return FeatureToggleDirective;
}());

var FeatureToggleProviderComponent = /** @class */ (function () {
    function FeatureToggleProviderComponent() {
        this.features = {};
        this.currentConfig = {};
    }
    FeatureToggleProviderComponent.prototype.ngOnInit = function () {
        if (typeof this.features !== 'object') {
            throw new Error('Attribute `features` should not be null or empty');
        }
        this.setFeatureToggles();
    };
    FeatureToggleProviderComponent.prototype.ngDoCheck = function () {
        this.setFeatureToggles();
    };
    FeatureToggleProviderComponent.prototype.setFeatureToggles = function () {
        if (this.currentConfig !== this.features) {
            // Using `Object.assign()` method for bundle size decreasing purposes
            // It's required since it needs a new memory reference
            // for the new object value
            this.currentConfig = Object.assign({}, this.features);
            set(this.features);
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FeatureToggleProviderComponent.prototype, "features", void 0);
    FeatureToggleProviderComponent = __decorate([
        Component({
            selector: 'feature-toggle-provider',
            template: '<ng-content></ng-content>'
        })
    ], FeatureToggleProviderComponent);
    return FeatureToggleProviderComponent;
}());

var FeatureToggleModule = /** @class */ (function () {
    function FeatureToggleModule() {
    }
    FeatureToggleModule = __decorate([
        NgModule({
            declarations: [FeatureToggleProviderComponent, FeatureToggleDirective],
            exports: [FeatureToggleProviderComponent, FeatureToggleDirective],
            imports: [CommonModule],
        })
    ], FeatureToggleModule);
    return FeatureToggleModule;
}());

var NgxFeatureToggleRouteGuard = /** @class */ (function () {
    function NgxFeatureToggleRouteGuard(router) {
        this.router = router;
    }
    NgxFeatureToggleRouteGuard.prototype.isDevMode = function () {
        return isDevMode();
    };
    NgxFeatureToggleRouteGuard.prototype.isOnCheck = function (route) {
        if (!route ||
            !route.data ||
            (typeof route.data.featureToggle !== 'string' &&
                !Array.isArray(route.data.featureToggle))) {
            if (this.isDevMode()) {
                console.error(
                // tslint:disable-next-line: max-line-length
                '`NgxFeatureToggleRouteGuard` need to receive `featureToggle` as data as an array or string in your route configuration.');
            }
            return false;
        }
        var hasAllTogglesOn = [].concat(route.data.featureToggle).every(function (toggle) {
            return toggle[0] === '!' ? !isOn(toggle.replace('!', '')) : isOn(toggle);
        });
        if (!hasAllTogglesOn && route.data.redirectTo) {
            this.router.navigate([].concat(route.data.redirectTo));
        }
        return hasAllTogglesOn;
    };
    NgxFeatureToggleRouteGuard.prototype.canLoad = function (route) {
        return this.isOnCheck(route);
    };
    NgxFeatureToggleRouteGuard.prototype.canActivateChild = function (route) {
        return this.isOnCheck(route);
    };
    NgxFeatureToggleRouteGuard.prototype.canActivate = function (route) {
        return this.isOnCheck(route);
    };
    NgxFeatureToggleRouteGuard.ctorParameters = function () { return [
        { type: Router }
    ]; };
    NgxFeatureToggleRouteGuard.ɵprov = ɵɵdefineInjectable({ factory: function NgxFeatureToggleRouteGuard_Factory() { return new NgxFeatureToggleRouteGuard(ɵɵinject(Router)); }, token: NgxFeatureToggleRouteGuard, providedIn: "root" });
    NgxFeatureToggleRouteGuard = __decorate([
        Injectable({ providedIn: 'root' }),
        __metadata("design:paramtypes", [Router])
    ], NgxFeatureToggleRouteGuard);
    return NgxFeatureToggleRouteGuard;
}());

/*
 * Public API Surface of ngx-feature-toggle
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FeatureToggleDirective, FeatureToggleModule, FeatureToggleProviderComponent, NgxFeatureToggleRouteGuard };
//# sourceMappingURL=ngx-feature-toggle.js.map
